/*  ---------------------------------------------------------------------------
    Program  :  mkFontTracer.sas
    Author   :  Perry Watts

    Project  :  CC32 

    Purpose  :  Make a FONT Tracer for the Container lineages in the SAS
                 Styles.Default Template

    Input    :  stylesDefault.txt
    Output   :  ContainerFontTracer.HTML
    
    Notes    :  *Created StylesDefault.txt with:
                    ods listing;
                      proc template;
                      source styles.default;
                    run;
                *Program is distributed on an "as is" basis.
    ------------------------------------------------------------------------  */
    %let Datpath=%str(C:\N10\CC\dat);
    %let odsPath=c:\n10\CC;

    data fonts1(keep=styleNum attributeNum style attribute);
      length style attribute $100;
      retain styleNum 0;
      retain attributeNum style; 
      infile "&Datpath.\stylesDefault.txt" length=linelen;
      input Aline $varying125. linelen;
      Aline=left(aline);
      if index(Aline,"style") eq 1 then do;
        styleNum+1;
        attributeNum=0;
        style=Aline;
      end;
      else do;
        if index(Aline,'"') ne 1 then do;
          attributeNum+1;
          attribute=lowCase(Aline);
          output;
        end;
      end;
    run;

    data fontCodes(keep=EquationLeft EquationRight)             /* ELEFT=FONTNAME, ERIGHT=FONT FOR PROC FORMAT */
         fontContainer(keep=styleNum styleName EquationRight);  /* ERIGHT=FONTNAME FOR FONTTBL */
      length styleName equationLeft equationRight $60;
      set fonts1;
      if styleNum EQ 1 OR (styleNum ge 10 AND styleNum LE 106);

      indexFrom=index(style,'from');
      if styleNum GE 1 AND attributeNum GE 1 then do;
        if indexFrom eq 0 then styleName=substr(style,7);
        else do;
          diff=indexFrom-8;
          styleName=substr(style,7,diff);
        end;
        attribute=compress(attribute,"';");
        indexEq=index(attribute,'=');
        EquationLeft=substr(attribute,1,indexEq-2);
        EquationRight=substr(attribute,indexEq+2);
      end;
      if styleNum GT 1 AND attributeNum GE 1 then do;
        if index(EquationRight,'(') then do;
          indexLparen=index(EquationRight,'(')+1;
          indexRparen=index(EquationRight,')')-1;
          diff=indexRParen-indexLparen+1;
          EquationRight=substr(EquationRight,indexLparen,diff);
        end;
      end;
      if styleNum Eq 1 then output fontCodes;
      else if EquationLeft EQ 'font' then output fontContainer;
    run;

  /* STYLENUM RETAINS THE HIERARCHY - THUS CONTAINER WILL BE FIRST IN THE LIST OF DOCFG DOCBG.*/
    proc sort data=fontContainer nodup;
      by EquationRight stylenum;
    run;

   data cntlinFCodes(keep=fmtname type start label hlo);
      retain type 'C' fmtname 'fontFm';
      length start $30 label $60;
      set fontCodes end=last;
      start=left(trim(EquationLeft));
      label=left(trim(EquationRight));
      output;
      if last then do;
        hlo='O';
        label=' ';
        output;
      end;
    run;  
    proc format library=work cntlin=cntlinFcodes;
    run;

   data fontTracer(keep=font fontName styleName);
     label font='Font' fontName='Font Name' styleName='Style Element';
     length font $60 fontName $30;
     set fontContainer;
     fontName=left(trim(equationRight));
     font=put(equationRight,$fontFm.);
     font=compress(font,"()");
    run;

   proc template;
    define style styles.fontStyle;
    parent=styles.default;
    replace output / 
         background = cxD9D9D9
         rules = ALL
         frame = BOX
         cellpadding = 3
         cellspacing = 2
         bordercolor = cx888888
         borderwidth = 3;
    replace data/
         foreground=black
         background=CXF8F8F8;
    replace header / 
         background=gray
         foreground=white
         font = (Arial,12pt,Bold);
    replace SystemTitle / 
         foreground=black
         font=("Arial",15pt,Bold);
    end; 
   run;
 
  proc sql noprint;
    create table uniqueFonts as 
      select distinct font from fontTracer;
    select count(*) into :n
      from uniqueFonts;
    select font into :f1 - :f%left(&n)
      from uniqueFonts;
  quit;
  %let n = %left(%trim(&n));

  options mprint;
  %macro tableTemplate;
  proc template;
     define Table TABLES.fontTbl;
       dynamic colhd;
       mvar %do i = 1 %to &n; f&i %end; ;
       classlevels=on;
       define column columnFont;
         blank_dups=on;
         define header colhd;
           style=header;
           just=Center;
         end;
         header=colhd;
         cellstyle 
           %do i= 1 %to %eval(&n - 1);
             _val_ EQ F&i as {font = (&&F&i)},
           %end;
           _val_ EQ F&n as {font = (&&F&n)};
       end;  /* DEFINE COLUMN */
       define column restOfData;
         generic=on;
         blank_dups=on;
         define header colhd;
           style=header;
           just=Center;
         end;
         header=colhd;
       end; /* DEFINE COLUMN */
     end;   /* DEFINE TABLE */
   run;
   %mend tableTemplate;
   %tableTemplate
  
   ods listing close;
   ods HTML
      style= styles.fontStyle
      path = "&odsPath" (URL=NONE)
      body = 'ContainerfontTracer.HTML'; 
   title1 "Font Tracer for the SAS STYLES.DEFAULT Container Style Elements";

     data _null_;
      set fontTracer;
       file print ods=(template='TABLES.fontTbl'
                      columns=(
                      columnfont=font(dynamic=(colhd='Font'))
                      restOfData=fontName(generic=on dynamic=(colhd='Font Name'))
                      restOfData=styleName(generic=on dynamic=(colhd='Style Element'))
                              )
                      );
      put _ods_;
     run;
   ods _all_ close;
   ods listing;
